﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using com.paytec.kit;
using System.Collections;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;

namespace KITTester
{
    public class DeviceCallbacks : com.paytec.kit.DeviceCallbacks
    {
        frmKITTester owner;

        public DeviceCallbacks(frmKITTester owner)
        {
            this.owner = owner;
        }

        public override void onCloseShiftResponse()
        {
            this.owner.BeginInvoke(new donCloseShiftResponse(_onCloseShiftResponse));
        }
        private delegate void donCloseShiftResponse();
        private void _onCloseShiftResponse()
        {
            owner.OutMessages.write("Terminal Response: Close shift");
        }

        public override void onAbortTransactionResponse()
        {
            this.owner.BeginInvoke(new donAbortTransactionResponse(_onAbortTransactionResponse));
        }
        private delegate void donAbortTransactionResponse();
        private void _onAbortTransactionResponse()
        {
            owner.OutMessages.write("Terminal Response: Abort Transaction");
        }

        public override void onConfigureResponse()
        {
            this.owner.BeginInvoke(new donConfigureResponse(_onConfigureResponse));
        }
        private delegate void donConfigureResponse();
        private void _onConfigureResponse()
        {
            owner.OutMessages.write("Terminal Response: Configure resposne");
        }

        public override void onConnectResponse()
        {
            this.owner.BeginInvoke(new donConnectResponse(_onConnectResponse));
        }
        private delegate void donConnectResponse();
        private void _onConnectResponse()
        {
            String deviceKey = owner.device.getDeviceId();
            

            owner.driverList.Add(deviceKey, owner.driver);
            owner.deviceList.Add(deviceKey, owner.device);
            owner.rebindDeviceComboBox();
            owner.lblTerminalID.Text = deviceKey;
            owner.firstConnect = false;
            
            
            owner.setDeviceState(true);
            owner.OutMessages.write("Terminal Response: Connect response");
        }

        public override void onDeviceStatusResponse(java.util.List l)
        {
            this.owner.BeginInvoke(new donDeviceStatusResponse(_onDeviceStatusResponse), l);
        }
        private delegate void donDeviceStatusResponse(java.util.List l);
        private void _onDeviceStatusResponse(java.util.List l)
        {
            owner.setDeviceState(l);
            owner.OutMessages.write("Terminal Response: Get status response");
        }

        public override void onDialogResponse(DialogResponse dr)
        {
            this.owner.BeginInvoke(new donDialogResponse(_onDialogResponse), dr);
        }
        private delegate void donDialogResponse(DialogResponse dr);
        private void _onDialogResponse(DialogResponse dr)
        {
            owner.OutMessages.write("Terminal Response:" + dr);
            if (dr.getEsign() != null)
            {
                owner.PicESignature.Image = Image.FromStream(new MemoryStream(dr.getEsign()));
            }
        }

        public override void onDoTransactionResponse(Transaction t)
        {
            this.owner.BeginInvoke(new donDoTransactionResponse(_onDoTransactionResponse), t);
        }
        private delegate void donDoTransactionResponse(Transaction t);
        private void _onDoTransactionResponse(Transaction t)
        {
            owner.OutMessages.write("Terminal Response: Transaction response:" + t);
            if (t.getESign() != null)
            {
                owner.PicESignature.Image = Image.FromStream(new MemoryStream(t.getESign()));
            }
            else
            {
                owner.PicESignature.Image = null;
            }

            try
            {
                String filename = "Transaktion_" + long.Parse(t.getTrxRefNum());

                // Serialization Tests 
                //outMessages.append("ORIG TRX: " + System.Environment.NewLine);
                //outMessages.append(transaction.toString());
                
                IFormatter formatter = new BinaryFormatter();
                Stream stream = new FileStream(filename, FileMode.Create, FileAccess.Write, FileShare.None);
                formatter.Serialize(stream, t);
                stream.Close();

                if (t.getTransactionFunction() == Transaction.Function.AUTHORIZATIONPURCHASE && t.getTransactionStatus() == Transaction.Status.APPROVED)
                {
                    Device d = (Device)t.getDevice();
                    String keyString = d.getDeviceId() + " - " + t.getTrxSeqCnt();
                    owner.transactionList.Add(keyString, t);
                    owner.rebindTransactionComboBox();
                }
            }
            catch (Exception ex)
            {
                //errMessages.append("ERROR in SERIALIZATION");
                //errMessages.append(ex.ToString());
            }
        }

        public override void onEjectCardResponse()
        {
            this.owner.BeginInvoke(new dOnEjectCardResponse(_onEjectCardResponse));
        }
        private delegate void dOnEjectCardResponse();
        private void _onEjectCardResponse()
        {
            owner.OutMessages.write("Terminal Response: Eject card");
        }

        public override void onError(Error __p1)
        {
            this.owner.BeginInvoke(new dOnError(_onError), __p1);
        }
        private delegate void dOnError(Error p1);
        private void _onError(Error error)
        {
            owner.OutMessages.write("Terminal Error:" + error.getErrorCode() + " " + error.getErrorMessage() + " at " + error.getErrorSource() + " native:" + error.getNativeMessage() + " native status:" + error.getNativeStatus());
        }

        public override void onFinalBalanceResponse()
        {
            this.owner.BeginInvoke(new dOnFinalBalanceResponse(_onFinalBalanceResponse));
        }
        private delegate void dOnFinalBalanceResponse();
        private void _onFinalBalanceResponse()
        {

        }

        public override void onInitializeResponse()
        {
            this.owner.BeginInvoke(new donInitializeResponse(_onInitializeResponse));
        }
        private delegate void donInitializeResponse();
        private void _onInitializeResponse()
        {

        }

        public override void onOpenShiftResponse()
        {
            this.owner.BeginInvoke(new donOpenShiftResponse(_onOpenShiftResponse));
        }
        private delegate void donOpenShiftResponse();
        private void _onOpenShiftResponse()
        {
            owner.OutMessages.write("Terminal Response: Open shift");
            owner.CmbTransaction.Items.Clear();
            Transaction.Function[] funcs = owner.device.getSupportedFunctions();
            for (int i = 0; i < funcs.Length; i++)
            {
                owner.CmbTransaction.Items.Add(funcs[i]);
            }
            owner.CmbTransaction.Sorted = true;
            owner.CmbTransaction.DisplayMember = "name";
            owner.CmbTransaction.SelectedItem = Transaction.Function.PURCHASE;

            Transaction.Currency[] currs = owner.device.getSupportedCurrencies();
            for (int i = 0; i < currs.Length; i++)
                owner.CmbCurrency.Items.Add(currs[i]);
            owner.CmbCurrency.Sorted = true;
            owner.CmbCurrency.DisplayMember = "name";
            owner.CmbCurrency.SelectedItem = Transaction.Currency.CHF;

        }

        public override void onStartUIExclResponse()
        {
            this.owner.BeginInvoke(new donStartUIExclResponse(_onStartUIExclResponse));
        }
        private delegate void donStartUIExclResponse();
        private void _onStartUIExclResponse()
        {

        }

        public override void onStopUIExclResponse()
        {
            this.owner.BeginInvoke(new donStopUIExclResponse(_onStopUIExclResponse));
        }
        private delegate void donStopUIExclResponse();
        private void _onStopUIExclResponse()
        {

        }

        public override void onChangeLanguageSelection(bool b)
        {
            this.owner.BeginInvoke(new donChangeLanguageSelection(_onChangeLanguageSelection), b);
        }
        private delegate void donChangeLanguageSelection();
        private void _onChangeLanguageSelection()
        {

        }

        public override void onReceiptResponse(Receipt receipt)
        {
            this.owner.BeginInvoke(new donReceiptResponse(_onReceiptResponse), receipt);
        }
        private delegate void donReceiptResponse(Receipt receipt);
        private void _onReceiptResponse(Receipt receipt)
        {
            owner.OutMessages.write("Receipt:\n" + receipt.getText());
        }

        public override void onReportResponse(Report report)
        {
            this.owner.BeginInvoke(new donReportResponse(_onReportResponse), report);
        }
        private delegate void donReportResponse(Report report);
        private void _onReportResponse(Report report)
        {
            foreach (ReportEntry entry in report.getReportEntries())
            {
                TreeNode node = new TreeNode(entry.getFinancialCounterType().toString());
                node.Nodes.Add(new TreeNode("Count:" + entry.getCount()));
                node.Nodes.Add(new TreeNode("Currency:" + entry.getCurrency()));
                node.Nodes.Add(new TreeNode("Total amount:" + entry.getTotalAmount()));
                node.Nodes.Add(new TreeNode("DCC Foreign Curr:" + entry.getDccForeignCurrency()));
                node.Nodes.Add(new TreeNode("DCC Foreing amount:" + entry.getDccForeignAmount()));
                node.Nodes.Add(new TreeNode("Brand:" + entry.getBrand()));
                node.Nodes.Add(new TreeNode("AID:" + entry.getAid()));

                owner.TrvReport.Nodes.Add(node);
            }
        }

        public override void onLoyaltyAdviceNotification(LoyaltyAdvice advice)
        {
            this.owner.BeginInvoke(new donLoyaltyAdviceNotification(_onLoyaltyAdviceNotification), advice);
        }
        private delegate void donLoyaltyAdviceNotification(LoyaltyAdvice advice);
        private void _onLoyaltyAdviceNotification(LoyaltyAdvice advice)
        {
            owner.OutMessages.append(advice.toString());
            owner.TrvReport.Nodes.Clear();

            TreeNode node;
            node = new TreeNode("Campaigns:");
            foreach (Campaign entry in advice.getCampaigns())
            {
                node.Nodes.Add(new TreeNode("Text:" + entry.toString()));
            }
            owner.TrvReport.Nodes.Add(node);

            node = new TreeNode("Redemption items:");
            foreach (RedemptionItem entry in advice.getRedemptionItems())
            {
                node.Nodes.Add(new TreeNode("Text:" + entry.toString()));

            }
            owner.TrvReport.Nodes.Add(node);
        }
    }
}
