﻿using System;
using System.Drawing;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;
using com.paytec.kit;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;


namespace KITTester
{
    public partial class frmKITTester : Form
    {

        public KITDriver driver;
        public Dictionary<String, KITDriver> driverList = new Dictionary<string, KITDriver>();
        public Device device;
        public Dictionary<String, Device> deviceList = new Dictionary<string, Device>();
        protected String configFile;
        public Boolean firstConnect = true;

        protected DeviceCallbacks devCbs;
        protected TransactionCallbacks trxCbs;
        protected System.IO.StringWriter swMessages;
        protected TextBoxStreamWriter errMessages;
        private TextBoxStreamWriter outMessages;
        protected Transaction transaction;
        public Dictionary<String, Transaction> transactionList = new Dictionary<String, Transaction>();



        public TextBoxStreamWriter OutMessages
        {
            get { return outMessages; }
            set { outMessages = value; }
        }

        java.util.ArrayList surrogatePANs = new java.util.ArrayList();

        public frmKITTester()
        {
            InitializeComponent();

            errMessages = new TextBoxStreamWriter(txtMessages, Color.Red);
            outMessages = new TextBoxStreamWriter(txtMessages, Color.Black);
        }

        public void setDeviceState(java.util.List l)
        {
            stLblActivated.Visible = l.contains(Device.State.ACTIVATED);

            stLblApplicationSelected.Visible = l.contains(Device.State.APPLICATION_SELECTED);

            stLblBusy.Visible = l.contains(Device.State.BUSY);

            stLblLocked.Visible = l.contains(Device.State.LOCKED);

            stLblReaderOccupied.Visible = l.contains(Device.State.READER_SLOT_OCCUPIED);

            stLblValidCard.Visible = l.contains(Device.State.VALID_CARD_INSERTED);

            pnlTransactionControls.Enabled = true;
            pnlCustomGUI.Enabled = stLblLocked.Visible && !stLblBusy.Visible;
            pnlCustomGUI.Enabled = true;

            pnlReporting.Enabled = !stLblLocked.Visible && !stLblBusy.Visible;

        }

        public void setDeviceState(bool connected)
        {
            if (!connected)
            {
                stLblConnected.Text = "Disconnected";
            }
            else
            {
                stLblConnected.Text = "Connected";
            }
        }

        private void frmKITTester_Load(object sender, EventArgs e)
        {
            System.Console.WriteLine(Environment.CurrentDirectory);

            this.Select();
            this.Show();

            cmbPOSEntryMode.DisplayMember = "name";

            cmbPOSEntryMode.Items.Add(Transaction.PosEntryMode.POS_ENTRY_MAGSTRIPE);
            cmbPOSEntryMode.Items.Add(Transaction.PosEntryMode.POS_ENTRY_MANUAL);
            cmbPOSEntryMode.Items.Add(Transaction.PosEntryMode.POS_ENTRY_UNSPECIFIED);

            cmbPOSEntryMode.SelectedItem = Transaction.PosEntryMode.POS_ENTRY_UNSPECIFIED;

            Trace.registerWriter(errMessages);

            cmbReceiptType.Items.Clear();
            foreach (Receipt.ReceiptType type in Receipt.ReceiptType.values())
            {
                cmbReceiptType.Items.Add(type);
            }

            cmbReportType.Items.Clear();
            foreach (Report.ReportType type in Report.ReportType.values())
            {
                cmbReportType.Items.Add(type);
            }

            cmbInputType.DataSource = Dialog.InputType.values();
            cmbDialogIcon.DataSource = Dialog.Icon.values();
            cmbDialogText.DataSource = Dialog.ResourceText.values();
            cmbDefaultScreenIcon.DataSource = com.paytec.kit.Screen.Icon.values();
            chklbDialogButtons.Items.Add(Dialog.Button.OK);
            chklbDialogButtons.Items.Add(Dialog.Button.Cancel);
            chklbDialogButtons.Items.Add(Dialog.Button.LeftCustom);
            chklbDialogButtons.Items.Add(Dialog.Button.RightCustom);

            chklbDialogFlags.Items.Add(Dialog.DialogFlags.NONE);
            chklbDialogFlags.Items.Add(Dialog.DialogFlags.LEAVE_ONSCREEN);
            chklbDialogFlags.Items.Add(Dialog.DialogFlags.WITH_DISPLAYTEXT);
        }

        private void btActivate_Click(object sender, EventArgs e)
        {
            device.openShift();
        }

        private void btDeactivate_Click(object sender, EventArgs e)
        {
            device.closeShift();
        }

        private void btFinalBalance_Click(object sender, EventArgs e)
        {
            device.finalBalance();
        }

        private void btDisconnect_Click(object sender, EventArgs e)
        {
            try
            {
                String deviceKey = device.getDeviceId();

                deviceList.Remove(deviceKey);
                driverList.Remove(deviceKey);
                device.disconnect();
                rebindDeviceComboBox();
                setDeviceState(false);

            }
            catch (Exception ex)
            {
                errMessages.append("Exception in DISCONNECT");
                errMessages.append(ex.ToString());
            }
        }

        private void btGetStatus_Click(object sender, EventArgs e)
        {
            device.requestDeviceStatus();
        }

        private void frmKITTester_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            { 
                foreach(KeyValuePair<String, Device> entry in deviceList)
                {
                    deviceList.Remove(entry.Key);
                    driverList.Remove(entry.Key);
                    entry.Value.disconnect();
                }
            }
            catch (Exception ex)
            {
                errMessages.append("Exception in DISCONNECT");
                errMessages.append(ex.ToString());
            }
            
        }

        private void btEjectCard_Click(object sender, EventArgs e)
        {
            device.ejectCard();
        }

        private void btDoTransaction_Click(object sender, EventArgs e)
        {

            switch (((Transaction.Function)cmbTransaction.SelectedItem).ordinal())
            {
                case (int)Transaction.Function.__Enum.PURCHASE:
                    transaction = Transaction.createPurchase((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.PURCHASEPHONEORDERED:
                    transaction = Transaction.createPurchasePhoneOrdered((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.PURCHASEMAILORDERED:
                    transaction = Transaction.createPurchaseMailOrdered((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.CREDIT:
                    transaction = Transaction.createCredit((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.PURCHASERESERVATION:
                    transaction = Transaction.createPurchaseReservation((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), txtTrxRefNr.Text);
                    break;
                case (int)Transaction.Function.__Enum.ACCOUNTINQUIRY:
                    transaction = Transaction.createAccountInquiry();
                    break;
                case (int)Transaction.Function.__Enum.AUTHORIZATIONPURCHASE:
                    transaction = Transaction.createAuthorizationPurchase((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.CASHADVANCE:
                    transaction = Transaction.createCashAdvance((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), true);
                    break;
                case (int)Transaction.Function.__Enum.CLIENTIDREQUEST:
                    transaction = Transaction.createClientIdRequest();
                    break;
                case (int)Transaction.Function.__Enum.COMBINEDTRANSACTION:
                    transaction = Transaction.createCombinedTransaction((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), false);
                    break;
                case (int)Transaction.Function.__Enum.CONFIRMPHONEAUTHORIZEDRESERVATION:
                    transaction = Transaction.createConfirmPhoneAuthorizedReservation((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.DEPOSIT:
                    transaction = Transaction.createDeposit((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), new String[3], true);
                    break;
                case (int)Transaction.Function.__Enum.GIRO:
                    transaction = Transaction.createGiro((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), false);
                    break;
                case (int)Transaction.Function.__Enum.PURCHASEFORCEDACCEPTANCE:
                    transaction = Transaction.createPurchaseForcedAcceptance((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.PURCHASEPHONEAUTHORIZED:
                    transaction = Transaction.createPurchasePhoneAuthorized((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), txtTrxRefNr.Text);
                    break;
                case (int)Transaction.Function.__Enum.RESERVATION:
                    transaction = Transaction.createReservation((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text));
                    break;
                case (int)Transaction.Function.__Enum.RESERVATIONADJUSTMENT:
                    transaction = Transaction.createReservationAdjustment((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), txtTrxRefNr.Text);
                    break;
                case (int)Transaction.Function.__Enum.REVERSAL:
                    transaction = Transaction.createReversal();
                    break;
                case (int)Transaction.Function.__Enum.PURCHASEWITHCASHBACK:
                    transaction = Transaction.createPurchaseWithCashback((Transaction.Currency)cmbCurrency.SelectedItem, long.Parse(txtAmount.Text), long.Parse(txtAmounttOther.Text));
                    break;
            }
            transaction.setPosEntryMode((Transaction.PosEntryMode)cmbPOSEntryMode.SelectedItem);
            switch (((Transaction.PosEntryMode)cmbPOSEntryMode.SelectedItem).ordinal())
            {
                case (int)Transaction.PosEntryMode.__Enum.POS_ENTRY_MAGSTRIPE:
                    break;
                case (int)Transaction.PosEntryMode.__Enum.POS_ENTRY_MANUAL:
                    transaction.setAppExpDate(DateTime.ParseExact(mtxtExpDate.Text, "MMyy", System.Globalization.CultureInfo.InvariantCulture).ToString("yyMMdd"));
                    transaction.setAppPAN(mtxtCardNumber.Text);
                    transaction.setCVC2(mtxtCVC2.Text);
                    break;
            }

            if ((sender as Button).Text == "Do transaction (silent)")
            {
                transaction.setTransactionRequestFlag(Transaction.TransactionRequestFlags.SILENT);
            }
            transaction.setDevice(device);
            device.doTransaction(transaction);

            /*
            String suPAN1 = ""; 
            String suPAN2 = "";

            surrogatePANs.add(HexToByte(suPAN1));
            surrogatePANs.add(HexToByte(suPAN2));

            
            if (surrogatePANs.size() > 0)
            {
                transaction.setListOfSurrogatePANs(surrogatePANs);
            }
            try
            {
                device.doTransaction(transaction);
            }
            catch (KITException ex)
            {
                errMessages.append("Do Transaction Exception:" + ex.getMessage());
            }
            */
        }

        private void txtMessages_TextChanged(object sender, EventArgs e)
        {
            txtMessages.ScrollToCaret();
        }

        private void btCommit_Click(object sender, EventArgs e)
        {
            if (chkCommitFromList.Checked && cmbAuthorizedTransactions.Items.Count > 0 && cbxDeSerialize.Checked == false)
            {
                transaction = null;
                String keyString = cmbAuthorizedTransactions.SelectedValue.ToString();
                transactionList.TryGetValue(keyString, out transaction);

            }

            if (cbxDeSerialize.Checked == true)
            {
                transaction = null;
            }

            if (transaction == null)
            {
                try
                {
                    String filename = "Transaktion_" + long.Parse(txtTrxSeqCnt.Text);
                    IFormatter formatter = new BinaryFormatter();
                    Stream stream = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read);
                    transaction = (Transaction)formatter.Deserialize(stream);
                    stream.Close();

                    //outMessages.append("DE-Serialized TRX: " + System.Environment.NewLine);
                    //outMessages.append(transaction.toString());
                }
                catch (Exception ex)
                {
                    errMessages.append("ERROR in DE-serialization");
                    errMessages.append(ex.ToString());
                }
            }

            if (!String.IsNullOrEmpty(txtAmountFinal.Text))
            {
                transaction.setAmountFinal(long.Parse(txtAmountFinal.Text));
            }
            try
            {
                transaction.setDevice(device);
                if (device.isConnected())
                {
                    transaction.commit();
                }
            }
            catch (Exception ex)
            {
                outMessages.append("Exception in COMMIT");
                outMessages.append(ex.ToString());
            }
        }

        private void btRollback_Click(object sender, EventArgs e)
        {
            transaction.rollback();
        }

        private void btAbort_Click(object sender, EventArgs e)
        {
            device.abortTransaction();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            device.abortTransaction(Device.AbortType.SILENT);
        }

        private void btInitialize_Click(object sender, EventArgs e)
        {
            device.initialize();
        }

        private void btConfigure_Click(object sender, EventArgs e)
        {
            device.configure();
        }

        private void btLockEFT_Click(object sender, EventArgs e)
        {
            device.startUIExcl();
        }

        private void btUnlockEFT_Click(object sender, EventArgs e)
        {
            device.stopUIExcl();
        }

        private void btLangSelOn_Click(object sender, EventArgs e)
        {
            device.enableLanguageSelection();
        }

        private void btLangSelOff_Click(object sender, EventArgs e)
        {
            device.disableLanguageSelection();
        }

        private void btGetReport_Click(object sender, EventArgs e)
        {
            device.requestReport((Report.ReportType)cmbReportType.SelectedItem);
        }

        private void btGetReceipt_Click(object sender, EventArgs e)
        {
            Int32 seqCnt = 0;
            if (Int32.TryParse(txtTrxSeqCnt.Text, out seqCnt))
            {
                device.requestReceipt((Receipt.ReceiptType)cmbReceiptType.SelectedItem, seqCnt);
            }
            else
            {
                device.requestReceipt((Receipt.ReceiptType)cmbReceiptType.SelectedItem, Receipt.receiptIDLastAvailable);
            }
        }

        private void btShowDialog_Click(object sender, EventArgs e)
        {
            long buttCode = 0;
            foreach (Object button in chklbDialogButtons.CheckedItems)
            {
                buttCode |= ((Dialog.Button)button).getCode();
            }
            long flagCode = 0;
            foreach (Object button in chklbDialogFlags.CheckedItems)
            {
                flagCode |= ((Dialog.DialogFlags)button).getCode();
            }

            string[] freitext = null;
            if (txtParameters.Text.Length > 0)
            {
                freitext = new string [] {txtParameters.Text};
            }
            


            com.paytec.kit.Dialog dlg = new com.paytec.kit.Dialog(
                (Dialog.InputType)cmbInputType.SelectedItem,
                (Dialog.Icon)cmbDialogIcon.SelectedItem,
                null,
                (Dialog.ResourceText)cmbDialogText.SelectedItem,
                freitext,
                long.Parse(txtDialogTimeout.Text),
                Dialog.Button.getByCode(buttCode),
                int.Parse(txtInputLengthMin.Text),
                int.Parse(txtInputLengthMax.Text),
                txtExpectedPassword.Text, txtWatermark.Lines, txtDefaultInputValue.Text);

            dlg.setDialogFlags(Dialog.DialogFlags.getByCode(flagCode));
            device.showDialog(dlg, null);
        }

        private void btCancelDialog_Click(object sender, EventArgs e)
        {
            device.abortDialog();
        }

        private void btClearBasket_Click(object sender, EventArgs e)
        {
            device.clearBasket();
        }

        private void txtMessages_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
                txtMessages.Clear();
        }

        private void lbBasketItemPalette_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void stLblConnected_Click(object sender, EventArgs e)
        {

        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {

        }

        private void stDeviceStates_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void label26_Click(object sender, EventArgs e)
        {

        }

        private void txtDeviceAddress_TextChanged(object sender, EventArgs e)
        {

        }

        private void cmbCurrency_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void label27_Click(object sender, EventArgs e)
        {

        }

        private void textAmtOther_TextChanged(object sender, EventArgs e)
        {

        }

        private void label28_Click(object sender, EventArgs e)
        {

        }

        private void button2_Click_1(object sender, EventArgs e)
        {
            if (String.Empty != textBox1.Text)
            {
                device.setTerminalLanguage(textBox1.Text);
            }

        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private static Byte[] HexToByte(String hexValue)
        {
            hexValue = Regex.Replace(hexValue, @"\s+", "");
            Int32 NumberChars = hexValue.Length;
            Byte[] bytes = new Byte[NumberChars / 2];
            for (Int32 i = 0; i < NumberChars; i += 2)
            {
                bytes[i / 2] = Convert.ToByte(hexValue.Substring(i, 2), 16);
            }

            return bytes;
        }

        public void rebindTransactionComboBox()
        {
            if (transactionList.Count > 0)
            {
                cmbAuthorizedTransactions.DataSource = new BindingSource(transactionList, null);
                cmbAuthorizedTransactions.DisplayMember = "Key";
                cmbAuthorizedTransactions.ValueMember = "Key";
                cmbAuthorizedTransactions.Update();
            }
            else
            {
                cmbAuthorizedTransactions.DataSource = null;
                cmbAuthorizedTransactions.Items.Clear();
                cmbAuthorizedTransactions.ResetText();
                cmbAuthorizedTransactions.Update();
                chkCommitFromList.Checked = false;
            }
        }

        public void rebindDeviceComboBox()
        {
            if (deviceList.Count > 0)
            {
                cmbDeviceList.DataSource = new BindingSource(deviceList, null);
                cmbDeviceList.DisplayMember = "Key";
                cmbDeviceList.ValueMember = "Key";
                cmbDeviceList.Update();
            }
            else
            {
                firstConnect = true;
                cmbDeviceList.DataSource = null;
                cmbDeviceList.Items.Clear();
                cmbDeviceList.ResetText();
                cmbDeviceList.Update();
            }
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (openFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    configFile = openFileDialog1.FileName;

                    driver = null;
                    device = null;

                    if (cmbLogLevel.Text != "0")
                    {
                        KITDriver.setDebug(true);
                        KITDriver.setDebugLevel(int.Parse(cmbLogLevel.Text));
                    }
                    else
                    {
                        KITDriver.setDebug(false);
                    }

                    driver = new com.paytec.kit.KITDriver(new DeviceCallbacks(this), new TransactionCallbacks(this), configFile);
                    device = driver.createDevice();
                    device.setHeartBeatWatchDogPeriod(50000);
                    device.connect();
                }
            }
            catch (Exception ex)
            {
                errMessages.append("Exception in CONNECT");
                errMessages.append(ex.ToString());
            }
        }

        private void cmbDeviceList_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (!firstConnect && cmbDeviceList.Items.Count > 0 && !String.IsNullOrEmpty(cmbDeviceList.SelectedValue.ToString()) && driverList.Count > 0 && deviceList.Count > 0)
                {
                    driver = null;
                    device = null;
                    driverList.TryGetValue(cmbDeviceList.SelectedValue.ToString(), out driver);
                    deviceList.TryGetValue(cmbDeviceList.SelectedValue.ToString(), out device);
                    lblTerminalID.Text = device.getDeviceId();
                    outMessages.append("SWITCHING to Terminal " + device.getDeviceId());
                    device.openShift();
                }
            }
            catch (Exception ex)
            {
                errMessages.append("Exception in switching Terminals");
                errMessages.append(ex.ToString());
            }
        }

        private void label9_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < chklbDialogButtons.Items.Count; i++)
            {
                
                chklbDialogButtons.SetItemChecked(i, !chklbDialogButtons.GetItemChecked(i));
            }
        }

        private void label14_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < chklbDialogFlags.Items.Count; i++)
            {
                chklbDialogFlags.SetItemChecked(i, !chklbDialogFlags.GetItemChecked(i));
            }
        }
    }
}

